"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const figures = require("figures");
const minimist = require("minimist");
const chalk_1 = require("chalk");
const exitHook = require("exit-hook");
const util_1 = require("util");
const logUpdate = require("log-update");
const Step_1 = require("./Step");
const argv = minimist(process.argv.slice(2));
const print = argv.hasOwnProperty('print') ? argv.print == 'true' : true;
const colors = argv.hasOwnProperty('colors') ? argv.colors == 'true' : process.stdout.isTTY;
const animate = argv.hasOwnProperty('animate') ? argv.animate == 'true' : process.stdout.isTTY;
const spinnerFrames = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏'];
class Renderer {
    static play(task) {
        if (!print) {
            return;
        }
        this._task = task;
        this._update();
        this._overrideConsoleLog();
        exitHook(this._exitHook.bind(this));
        setInterval(() => this._update(), 120).unref();
    }
    static _update() {
        let text = '';
        text += '\n';
        text += this._getTextAtLevel(this._task, 0);
        text += this._getErrorsText();
        if (this._logs.length) {
            text += '\n';
            text += this._logs.join('\n');
        }
        if (!text.endsWith('\n')) {
            text += '\n';
        }
        if (this._lastRenderedText != text) {
            logUpdate(text);
        }
        this._lastRenderedText = text;
        this._spinnerFrameIndex = this._spinnerFrameIndex < spinnerFrames.length - 1
            ? this._spinnerFrameIndex + 1
            : 0;
    }
    static _getErrorsText() {
        let text = '';
        let queue = [...this._task.steps];
        while (queue.length) {
            let step = queue.pop();
            if (step.error) {
                text += util_1.inspect(step.error, { colors: true });
            }
            else if (step.errorObject) {
                text += util_1.inspect(step.errorObject, { colors: true });
            }
            if (step.child) {
                queue.push(...step.child.steps);
            }
        }
        return text;
    }
    static _getTextAtLevel(task, level) {
        let text = '';
        task.steps.forEach(step => {
            if (step.name) {
                text += ' '.repeat(level * 2);
                text += this._getStepText(step);
                text += '\n';
            }
            if (step.child) {
                text += this._getTextAtLevel(step.child, step.name ? level + 1 : level);
            }
        });
        return text;
    }
    static _getStepText(step) {
        let text = '';
        switch (step.status) {
            case Step_1.StepStatus.Running:
                text += process.platform == 'win32' || !animate
                    ? this._chalk.yellow(figures.play)
                    : this._chalk.yellow(spinnerFrames[this._spinnerFrameIndex]);
                break;
            case Step_1.StepStatus.Success:
                text += this._chalk.green(figures.tick);
                break;
            case Step_1.StepStatus.Failure:
                text += this._chalk.red(figures.cross);
                break;
        }
        text += ' ';
        text += step.name;
        if (step.status == Step_1.StepStatus.Failure && step.errorMessage) {
            text += ' ' + figures.arrowRight + ' ' + this._chalk.red(step.errorMessage);
        }
        else if (step.info && (step.status == Step_1.StepStatus.Running || step.status == Step_1.StepStatus.Failure)) {
            text += ' ' + figures.arrowRight + ' ' + this._chalk.yellow(step.info);
        }
        return text;
    }
    static _exitHook() {
        this._update();
        logUpdate.done();
    }
    static _overrideConsoleLog() {
        console.log = (...args) => {
            let toString = (value) => {
                if (typeof value == 'string') {
                    return util_1.format(value);
                }
                else {
                    return util_1.inspect(value, { colors: true });
                }
            };
            this._logs.push(args.map(value => toString(value)).join(' '));
        };
    }
}
Renderer._logs = [];
Renderer._lastRenderedText = '';
Renderer._spinnerFrameIndex = 0;
Renderer._chalk = new chalk_1.default.constructor({ enabled: colors });
exports.Renderer = Renderer;
//# sourceMappingURL=Renderer.js.map