"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const minimist = require("minimist");
const Renderer_1 = require("./Renderer");
const Step_1 = require("./Step");
class Task {
    constructor() {
        this.steps = [];
        this._options = {};
        this._stateValues = {};
        process.on('uncaughtException', (err) => this._errorOut(this._getCurrentStep(), err));
        process.on('unhandledRejection', (message) => this._errorOut(this._getCurrentStep(), message));
    }
    add(value) {
        if (value instanceof Task) {
            this.steps.push(new Step_1.Step({
                child: value
            }));
        }
        else if (value instanceof Function) {
            this.steps.push(new Step_1.Step({
                exec: value
            }));
        }
        else {
            this.steps.push(new Step_1.Step(value));
        }
        return this;
    }
    withOptions(options) {
        let copy = new Task();
        copy._options = options;
        copy.steps = this.steps;
        copy._stateValues = this._stateValues;
        return copy;
    }
    run() {
        Renderer_1.Renderer.play(this);
        return this._execTasks(this);
    }
    _errorOut(step, err) {
        process.exitCode = 1;
        if (step) {
            step.failure(err);
        }
        else if (util_1.types.isNativeError(err)) {
            throw err;
        }
    }
    _getCurrentStep() {
        let queue = this.steps.slice();
        while (queue.length) {
            let step = queue.pop();
            if (step.status == Step_1.StepStatus.Running || step.status == Step_1.StepStatus.Failure) {
                if (step.child) {
                    queue.push(...step.child.steps);
                }
                else {
                    return step;
                }
            }
        }
    }
    async _execTasks(task) {
        for (let i = 0; i < task.steps.length; i++) {
            await this._execTask(task.steps[i], task._options);
        }
    }
    _execTask(step, options) {
        return new Promise((resolve, reject) => {
            step.start();
            let done = () => {
                step.success();
                resolve();
            };
            let result;
            if (step.exec) {
                try {
                    result = step.exec(this._generateState(options));
                }
                catch (err) {
                    this._errorOut(step, err);
                }
            }
            if (step.status == Step_1.StepStatus.Failure) {
                reject();
            }
            else if (step.concurrent) {
                resolve();
            }
            else if (result && result.then instanceof Function && result.catch instanceof Function) {
                result.then(done).catch((err) => this._errorOut(step, err));
            }
            else if (step.child) {
                this._execTasks(step.child).then(done);
            }
            else {
                done();
            }
        });
    }
    _generateState(options) {
        let index = process.argv.indexOf('--');
        let argv;
        if (index == -1) {
            argv = minimist([]);
        }
        else {
            argv = minimist(process.argv.slice(index + 1));
        }
        return {
            argv: argv,
            options: options,
            get: (key) => this._stateValues[key],
            set: (key, value) => this._stateValues[key] = value,
            info: (message) => {
                this._getCurrentStep().info = message;
            },
            fail: (message) => {
                this._errorOut(this._getCurrentStep(), message);
            }
        };
    }
}
exports.default = Task;
//# sourceMappingURL=Task.js.map